# -*- coding: utf-8 -*-

import requests
from requests.adapters import HTTPAdapter
from urllib3.util import Retry


class NofraudApi():
  production_url = 'https://api.nofraud.com/'
  portal_url = 'https://portal-api.nofraud.com/'
  sandbox_url    = 'https://apitest.nofraud.com/'

  def __init__(self, api_token, sandbox_mode=True, timeout=60):
    self.api_token = api_token
    self.sandbox_mode = sandbox_mode
    self.timeout = timeout

    retry_strategy = Retry(
      total=7,  # Maximum number of retries
      status_forcelist=[429, 500, 502, 503, 504],  # HTTP status codes to retry on
    )
    adapter = HTTPAdapter(max_retries=retry_strategy)
    self.client = requests.Session()
    self.client.mount('http://', adapter)
    self.client.mount('https://', adapter)

  def send(self, url, method='get', body=None, headers=None):
    return self.client.request(method.upper(), url, json=body, headers=headers, timeout=self.timeout)

  def send_screen_request(self, body, headers=None):
    url = self.sandbox_url if self.sandbox_mode else self.production_url
    if body and 'nTtoken' not in body:
      body['nfToken'] = self.api_token

    return self.send(url, 'post', body, headers)

  def get_transaction_status(self, transaction_id):
    host = self.sandbox_url if self.sandbox_mode else self.production_url
    url = '{}status_by_url/{}/{}'.format(host, self.api_token, transaction_id)

    return self.send(url, 'get')

  def cancel_transaction_request(self, transaction_id):
    host = self.sandbox_url if self.sandbox_mode else self.production_url
    url = "{}api/v1/transaction-update/cancel-transaction".format(host)
    body = {"nf_token": self.api_token, "transaction_id": transaction_id}

    return self.send(url, 'post', body)

  def report_fraud_request(self, transaction_id):
    host = self.sandbox_url if self.sandbox_mode else self.production_url
    body = {"nf_token": self.api_token, "transaction_id": transaction_id}
    url = "{}api/v1/transaction-update/report-fraud".format(host)

    return self.send(url, 'post', body)
