# -*- coding: utf-8 -*-

import pdb

from em_tasks import logger
from em_tasks.nofraud.api import NofraudApi


class ShopifyUtil():
  def __init__(self, api_token, sandbox_mode=True):
    self.api = NofraudApi(api_token, sandbox_mode)

  def screen_shopify_order(self, shopify_order):
    payload = None
    try:
      payload = self.to_nofraud_order(shopify_order)
      if not payload:
        return
    except Exception as e:
      logger.exception(e)

    if not payload:
      # pdb.set_trace()
      return

    resp = self.api.send_screen_request(payload)
    nf_result = resp.json()
    if 'id' not in nf_result:
      logger.error('[NofraudError] Payload: %s, Error: %s', payload, nf_result)
      return

    return {
      'transaction_id': nf_result['id'],
      'decision': nf_result['decision'],
      'message': nf_result.get('message', None)
    }

  def to_nofraud_order(self, shopify_order):
    if shopify_order.test:
      return

    if shopify_order.cancelled_at:
      return

    shipping_charge = round(sum([float(shipping_line.discounted_price) for shipping_line in shopify_order.shipping_lines]), 2)
    transactions = shopify_order.transactions()
    payment = None
    for transaction in transactions:
      if transaction.status == 'success':
        payment = transaction.payment_details
        break
    if not payment:
      return

    customer = shopify_order.customer

    base_params = {
      'amount': str(shopify_order.total_price),
      'currencyCode': shopify_order.currency,
      'shippingAmount': str(shipping_charge),
      'customerIP': shopify_order.browser_ip,
      'avsResultCode': payment.avs_result_code,
      'cvvResultCode': payment.cvv_result_code,
    }
    if not base_params['avsResultCode']:
      return

    if not base_params['cvvResultCode']:
      base_params['cvvResultCode'] = 'Y'

    customer_params = {
      'email': shopify_order.email,
    }
    orders_count = getattr(customer, 'orders_count', None)
    if orders_count:
      customer_params['totalPreviousPurchases'] = orders_count

    total_spent = getattr(customer, 'total_spent', None)
    if total_spent:
      customer_params['totalPurchaseValue'] = total_spent

    order_params = {
      'invoiceNumber': shopify_order.name
    }

    payment_params = {
      'method': 'Credit Card',
      'creditCard': {
        'last4': payment.credit_card_number[-4:],
        'cardType': payment.credit_card_company,
        'expirationDate': '{}/{}'.format("{0:0>2}".format(payment.credit_card_expiration_month), payment.credit_card_expiration_year)
      }
    }

    sa = shopify_order.shipping_address
    address = sa.address1
    if sa.address2:
      address = ' '.join([sa.address1, sa.address2])

    ship_address_params = {
      'firstName': sa.first_name,
      'lastName': sa.last_name,
      'address': address,
      'city': sa.city,
      'state': sa.province_code,
      'zip': sa.zip,
      'country': sa.country_code,
    }
    if sa.phone:
      ship_address_params['phoneNumber'] = sa.phone

    ba = shopify_order.billing_address
    address = ba.address1
    if ba.address2:
      address = ' '.join([ba.address1, ba.address2])
    bill_address_params = {
      'firstName': ba.first_name,
      'lastName': ba.last_name,
      'address': address,
      'city': ba.city,
      'state': ba.province_code,
      'zip': ba.zip,
      'country': ba.country_code,
    }
    if ba.phone:
      ship_address_params['phoneNumber'] = ba.phone

    # lis = dict()
    # for line_item in shopify_order.line_items:
    #   if line_item.sku in lis:
    #     li_attr = lis[line_item.sku]
    #     li_attr['price'] += line_item.price
    #     li_attr['quantity'] += line_item.quantity
    #   else:
    #     lis[line_item.sku] = {
    #       'sku': line_item.sku,
    #       'name': line_item.name[:100],
    #       'price': line_item.price,
    #       'quantity': line_item.quantity
    #     }
    # line_items_params = list(lis.values())

    line_items_params = []
    for line_item in shopify_order.line_items:
      line_items_params.append({
        'sku': line_item.sku,
        'name': line_item.name[:100],
        'price': line_item.price,
        'quantity': line_item.quantity
      })

    nofraud_order = dict(base_params)
    nofraud_order['customer'] = customer_params
    nofraud_order['order'] = order_params
    nofraud_order['payment'] = payment_params
    nofraud_order['shipTo'] = ship_address_params
    nofraud_order['billTo'] = bill_address_params
    nofraud_order['lineItems'] = line_items_params

    return nofraud_order