# -*- coding: utf-8 -*-

import time
import datetime

import dateutil
import dateutil.parser

from em_tasks import logger
from em_tasks.spapi.spapi_catalog_items_parser import SpapiCatalogItemsParser


class SpapiUpdateCatalogItemsTask():
  def __init__(self, spapi, product_service, marketplace, asins):
    self.spapi = spapi
    self.product_service = product_service
    self.marketplace = marketplace.lower()
    self.asins = asins

  def run(self):
    cur_time = datetime.datetime.strftime(datetime.datetime.utcnow(), '%Y-%m-%dT%H:%M:%S')
    indice_name = 'amz_products_api_{}'.format(self.marketplace)

    products_info = self.search_catalog_items(self.spapi, self.marketplace, self.asins)
    if not products_info:
      return

    for _, product_info in products_info.items():
      product_info['_id'] = product_info['asin']
      product_info['time'] = cur_time
    try:
      self.product_service.save_products(indice_name, list(products_info.values()))
    except Exception as e:
      logger.warning('[ProductSaveToServiceError] %s', products_info)
      logger.exception(e)

  def search_catalog_items(self, spapi, marketplace, asins):
    response = spapi.search_catalog_items(asins, marketplace=marketplace)
    if not response:
      return

    return SpapiCatalogItemsParser.parse(response)
