# -*- coding: utf-8 -*-

import re
import datetime

import shopify
import pyactiveresource
import shopify_python_api_patcher
from shopify_python_api_patcher.utils import get_current_release

from gspread.exceptions import APIError
from tracking_numbers import get_tracking_number

from em_tasks import logger
from em_tasks.tools.shopify_orders_sheet_base import ShopifyOrdersSheetBase
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.utils.order_reader import GSheetOrderReader
from em_tasks.store.models import *


class ShopifyOrdersSheetProductUrlsCorrector(ShopifyOrdersSheetBase):
  def __init__(self, service_account_data, days):
    self.service_account_data = service_account_data
    self.sheet_api = SheetAPI(service_account_data)
    self.order_reader = GSheetOrderReader(self.sheet_api)
    self.days = days

  def run(self):
    ss_ids = dict()
    for items in self.__class__.spreadsheet_ids.values():
      for spreadsheet_id in items.values():
        ss_ids[spreadsheet_id] = None

    for spreadsheet_id in ss_ids.keys():
      for worksheet, headers, orders in self.order_reader.read(spreadsheet_id, self.days):
        sheet_orders = dict()
        records = []
        for order in orders:
          order_id = order.get('order-id', '')
          if not order_id:
            continue

          order_id_valid = True
          for part in order_id.split('-'):
            if not part.isnumeric():
              order_id_valid = False
              break
          if not order_id_valid:
            continue

          product_link = order.get('sku address', None)
          if product_link:
            continue

          s_order_id = order_id.split('-').pop(0)
          order['s_order_id'] = s_order_id

          sheet_orders[order_id] = {
            'order': order,
            'headers': headers,
            'worksheet': worksheet
          }

        s_order_ids = list({sheet_order['order']['s_order_id']:None for sheet_order in sheet_orders.values()}.keys())
        for order_item in OrderItem.select().where(OrderItem.s_order_id.in_(s_order_ids)).execute():
          k = '{}-{}'.format(order_item.s_order_id, order_item.s_line_item_id)
          if k not in sheet_orders:
            continue

          product_link = order_item.s_url
          if not product_link:
            continue
          s_order = sheet_orders[k]
          s_order_data = s_order['order']
          headers = s_order['headers']
          worksheet = s_order['worksheet']

          try:
            product_link_h = 'sku address'
            if product_link_h not in headers:
              continue
            sku_col = headers.index('order-id') + 1
            product_link_col = headers.index(product_link_h) + 1
            product_link_col_offset = product_link_col - sku_col

            cell = self.sheet_api.find_cell_by_key(worksheet, s_order_data['order-id'])
            row = cell.row
            pl_col = cell.col + product_link_col_offset
            self.sheet_api.update_worksheet_cell(worksheet, row, pl_col, product_link)

            logger.info(
              '[SheetOrderUpdated] SheetName: %s, OrderID: %s, OrderNumber: %s, ProductLink: %s',
              s_order_data['sheet'], s_order_data['order-id'], s_order_data['order_number'], product_link)
          except:
            logger.warning('[SheetOrderError] SheetName: %s, OrderID: %s, OrderNumber: %s',
              s_order_data['sheet'], s_order_data['order-id'], s_order_data['order_number'])
