import time

from dropshipping.utils.offer_converters import EsLowestOfferListingOfferConverter
from dropshipping.utils.offer_filters import LowestOfferListingOfferFilter
from dropshipping.utils.offer_services import EsLowestOfferListingOfferService
from dropshipping.utils.price_finders import OfferServiceLowestOfferListingPriceFinder

from em_tasks import logger


class AmzOfferService():
  def __init__(self, host, port, user, password, filter_cond):
    self.filter_cond = filter_cond
    offer_filter = LowestOfferListingOfferFilter(self.filter_cond)
    offer_converter = EsLowestOfferListingOfferConverter()

    es_offer_service = EsLowestOfferListingOfferService(host, port, user, password)

    self.es_finder = OfferServiceLowestOfferListingPriceFinder(
      offer_service=es_offer_service, offer_filter=offer_filter, offer_converter=offer_converter)

  def get_offers(self, marketplace, asins, condition='new'):
    offers = False
    max_retries = 3
    while max_retries > 0:
      try:
        offers = self.es_finder.find_offer_for_asins(asins, country=marketplace, condition=condition)
        if not isinstance(offers, dict):
          logger.info('[OfferService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    return offers
