# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class RakutenUploadableProductsFilter():
  def __init__(self, product_service):
    self.product_service = product_service
    self.indice_name = 'rakuten_products'

  def filter(self):
    blacklisted_path = './rakuten_blacklisted.txt'
    records = dict()
    if os.path.isfile(blacklisted_path):
      with open(blacklisted_path, encoding='utf-8', errors='ignore') as fh:
        for line in fh:
          records[line.strip()] = None
    else:
      for keyword in BlacklistKeyword.select().order_by(BlacklistKeyword.id.desc()).dicts():
        k = keyword['keywords']
        records[k]= []

        query = {
          'query': {
            'query': {
              'multi_match': {
                'query': k,
                'type': 'phrase',
                'fields': ['title^10', 'description']
              }
            }
          }
        }
        for prod_id, _ in self.product_service.load_products(self.indice_name, query):
          records[prod_id] = None

      with open('./ebay_blacklisted.txt', 'w+', encoding='utf-8', errors='ignore') as fh:
        fh.write("\n".join(list(records.keys())))

    logger.debug('[BlacklistedProductsLoaded] ProductsCnt: %s', len(records))

    uploaded_prods = dict()
    for prod in StoreUtil.get_rakuten_products():
      uploaded_prods[prod['source_product_id']] = None

    logger.debug('[UploadedProductsLoaded] ProductsCnt: %s', len(uploaded_prods))

    for prod_id, prod in self.product_service.load_products(self.indice_name):
      exist = prod.get('existence', True)
      if not exist:
        continue

      if prod_id in records or prod_id in uploaded_prods:
        continue

      yield (prod_id, prod)
