# -*- coding: utf-8 -*-

import datetime

import shopify

from em_tasks import logger
from em_tasks.nofraud.shopify_util import ShopifyUtil
from em_tasks.store.models import *


class ShopifyNofraudScanner():
  def __init__(self, api_token, days=14):
    self.shopify_util = ShopifyUtil(api_token, False)
    self.days = 14

  def run(self):
    today = datetime.datetime.utcnow()
    from_date = today - datetime.timedelta(days=self.days)
    order_ids = dict()
    for order_item in OrderItem.select().where((OrderItem.risky_action == 'investigate') & (OrderItem.s_purchase_date > from_date)).execute():
      order_ids[order_item.s_order_id] = None

    for order_nofraud in OrderNofraud.select().where(OrderNofraud.s_order_id.in_(list(order_ids.keys()))).execute():
      order_ids.pop(order_nofraud.s_order_id, None)

    if not order_ids:
      return

    order_ids = list(order_ids.keys())
    page_size = 250
    chunks = [order_ids[x:x + page_size] for x in range(0, len(order_ids), page_size)]
    for chunk in chunks:
      self.process_products(chunk)

  def process_products(self, order_ids):
    records = []
    try:
      oids = ','.join([str(oid) for oid in order_ids])
      shopify_orders = shopify.Order.find(ids=oids, limit=len(order_ids))
      for shopify_order in shopify_orders:
        res = self.shopify_util.screen_shopify_order(shopify_order)
        if not res:
          continue

        record = {
          's_order_id': shopify_order.id,
          's_order_number': shopify_order.name,
        }
        record.update(res)
        logger.info('[ShopifyOrderScanned] %s', record)

        records.append(record)
    finally:
      if records:
        OrderNofraud.insert_many(records).execute()
